@@ -156,6 +156,7 @@ up_daemon_update_display_battery (UpDaemon *daemon)
 
 		UpDeviceState state = UP_DEVICE_STATE_UNKNOWN;
 		UpDeviceKind kind = UP_DEVICE_KIND_UNKNOWN;
+		gboolean present = FALSE;
 		gdouble percentage = 0.0;
 		gdouble energy = 0.0;
 		gdouble energy_full = 0.0;
@@ -166,6 +167,7 @@ up_daemon_update_display_battery (UpDaemon *daemon)
 
 		device = g_ptr_array_index (array, i);
 		g_object_get (device,
+			      "is-present", &present,
 			      "type", &kind,
 			      "state", &state,
 			      "percentage", &percentage,
@@ -177,6 +179,9 @@ up_daemon_update_display_battery (UpDaemon *daemon)
 			      "power-supply", &power_supply,
 			      NULL);
 
+		if (!present)
+			continue;
+
 		/* When we have a UPS, it's either a desktop, and
 		 * has no batteries, or a laptop, in which case we
 		 * ignore the batteries */
@@ -367,12 +372,15 @@ up_daemon_get_on_ac_local (UpDaemon *daemon, gboolean *has_ac)
 	gboolean ret;
 	gboolean result = FALSE;
 	gboolean online;
+	UpDaemonPrivate *priv;
 	UpDevice *device;
 	GPtrArray *array;
 
 	if (has_ac)
 		*has_ac = FALSE;
 
+	priv = up_daemon_get_instance_private (daemon);
+
 	/* ask each device */
 	array = up_device_list_get_array (daemon->priv->power_devices);
 	for (i=0; i<array->len; i++) {
@@ -385,6 +393,14 @@ up_daemon_get_on_ac_local (UpDaemon *daemon, gboolean *has_ac)
 			break;
 		}
 	}
+
+	/* if battery is discharging and AC is online, return FALSE
+	   since a low power charger maybe plugged to the system */
+	if (priv->kind == UP_DEVICE_KIND_BATTERY &&
+	    priv->state == UP_DEVICE_STATE_DISCHARGING &&
+	    online)
+		result = FALSE;
+
 	g_ptr_array_unref (array);
 	return result;
 }
